@@ -7,6 +7,7 @@
 #include <qpdf/Constants.h>
 #include <qpdf/Types.h>
 #include <qpdf/DLL.h>
+#include <qpdf/Pl_String.hh>
 #include <qpdf/QPDFExc.hh>
 #include <qpdf/QPDFObjGen.hh>
 #include <qpdf/QPDFXRefEntry.hh>
@@ -60,138 +61,6 @@ py::size_t list_range_check(QPDFObjectHandle h, int index)
     return static_cast<py::size_t>(index);
 }
 
-inline bool typecode_is_bool(qpdf_object_type_e typecode)
-{
-    return typecode == qpdf_object_type_e::ot_boolean;
-}
-
-inline bool typecode_is_int(qpdf_object_type_e typecode)
-{
-    return typecode == qpdf_object_type_e::ot_integer;
-}
-
-inline bool typecode_is_numeric(qpdf_object_type_e typecode)
-{
-    return typecode == qpdf_object_type_e::ot_integer ||
-           typecode == qpdf_object_type_e::ot_real ||
-           typecode == qpdf_object_type_e::ot_boolean;
-}
-
-bool objecthandle_equal(QPDFObjectHandle self, QPDFObjectHandle other)
-{
-    StackGuard sg(" objecthandle_equal");
-
-    // Uninitialized objects are never equal
-    if (!self.isInitialized() || !other.isInitialized())
-        return false; // LCOV_EXCL_LINE
-
-    // If two objects point to the same underlying object, they are equal (in fact,
-    // they are identical; they reference the same underlying QPDFObject, even if the
-    // handles are different). This lets us compare deeply nested and cyclic
-    // structures without recursing into them.
-    if (self.isSameObjectAs(other)) {
-        return true;
-    }
-
-    auto self_typecode  = self.getTypeCode();
-    auto other_typecode = other.getTypeCode();
-
-    if (typecode_is_bool(self_typecode) && typecode_is_bool(other_typecode)) {
-        return self.getBoolValue() == other.getBoolValue();
-    } else if (typecode_is_int(self_typecode) && typecode_is_int(other_typecode)) {
-        return self.getIntValue() == other.getIntValue();
-    } else if (typecode_is_numeric(self_typecode) &&
-               typecode_is_numeric(other_typecode)) {
-        // If 'self' and 'other' are different numeric types, convert both to
-        // Decimal objects and compare them as such.
-        auto a              = decimal_from_pdfobject(self);
-        auto b              = decimal_from_pdfobject(other);
-        py::object pyresult = a.attr("__eq__")(b);
-        bool result         = pyresult.cast<bool>();
-        return result;
-    }
-
-    // Apart from numeric types, dissimilar types are never equal
-    if (self_typecode != other_typecode)
-        return false;
-
-    switch (self_typecode) {
-    case qpdf_object_type_e::ot_null:
-        return true; // Both must be null
-    case qpdf_object_type_e::ot_name:
-        return self.getName() == other.getName();
-    case qpdf_object_type_e::ot_operator:
-        return self.getOperatorValue() == other.getOperatorValue();
-    case qpdf_object_type_e::ot_string: {
-        // We don't know what encoding the string is in
-        // This ensures UTF-16 coded ASCII strings will compare equal to
-        // UTF-8/ASCII coded.
-        return self.getStringValue() == other.getStringValue() ||
-               self.getUTF8Value() == other.getUTF8Value();
-    }
-    case qpdf_object_type_e::ot_array: {
-        // Call operator==() on each element of the arrays, meaning this
-        // recurses into this function
-        if (self.getArrayNItems() != other.getArrayNItems())
-            return false;
-        {
-            auto self_aitems  = self.aitems();
-            auto other_aitems = other.aitems();
-            auto iter_self    = self_aitems.begin();
-            auto iter_other   = other_aitems.begin();
-            for (; iter_self != self_aitems.end(); ++iter_self, ++iter_other) {
-                if (!objecthandle_equal(*iter_self, *iter_other))
-                    return false;
-            }
-        }
-        return true;
-    }
-    case qpdf_object_type_e::ot_dictionary: {
-        // Call operator==() on each element of the arrays, meaning this
-        // recurses into this function
-        return (self.getDictAsMap() == other.getDictAsMap());
-    }
-    case qpdf_object_type_e::ot_stream: {
-        // Recurse into this function to check if our dictionaries are equal
-        if (!objecthandle_equal(self.getDict(), other.getDict()))
-            return false;
-
-        // If dictionaries are equal, check our stream
-        // We don't go as far as decompressing the data to see if it's equal
-        auto self_buffer  = self.getRawStreamData();
-        auto other_buffer = other.getRawStreamData();
-
-        // Early out: if underlying qpdf Buffers happen to be the same, the data is
-        // the same
-        if (self_buffer == other_buffer)
-            return true;
-        // Early out: if sizes are different, data cannot be the same
-        if (self_buffer->getSize() != other_buffer->getSize())
-            return false;
-
-        // Slow path: memcmp the binary data
-        return 0 == std::memcmp(self_buffer->getBuffer(),
-                        other_buffer->getBuffer(),
-                        self_buffer->getSize());
-    }
-    // LCOV_EXCL_START
-    case qpdf_object_type_e::ot_boolean:
-    case qpdf_object_type_e::ot_integer:
-    case qpdf_object_type_e::ot_real:
-        throw std::logic_error("should have eliminated numeric types by now");
-    default:
-        throw std::logic_error("invalid object type");
-    }
-    // LCOV_EXCL_STOP
-}
-
-bool operator==(QPDFObjectHandle self, QPDFObjectHandle other)
-{
-    // A lot of functions in QPDFObjectHandle are not tagged const where they
-    // should be, but are const-safe
-    return objecthandle_equal(self, other);
-}
-
 bool object_has_key(QPDFObjectHandle h, std::string const &key)
 {
     if (!h.isDictionary() && !h.isStream())
@@ -302,7 +171,7 @@ void init_object(py::module_ &m)
         .value("operator", qpdf_object_type_e::ot_operator)
         .value("inlineimage", qpdf_object_type_e::ot_inlineimage);
 
-    py::class_<Buffer, std::shared_ptr<Buffer>>(m, "Buffer", py::buffer_protocol())
+    py::class_<Buffer, py::smart_holder>(m, "Buffer", py::buffer_protocol())
         .def_buffer([](Buffer &b) -> py::buffer_info {
             return py::buffer_info(b.getBuffer(),
                 sizeof(unsigned char),
@@ -336,7 +205,7 @@ void init_object(py::module_ &m)
 #if _MSC_VER
 #    pragma warning(suppress : 4267)
 #endif
-    py::class_<QPDFObjectHandle>(m, "Object")
+    py::class_<QPDFObjectHandle, py::smart_holder>(m, "Object")
         .def_property_readonly("_type_code", &QPDFObjectHandle::getTypeCode)
         .def_property_readonly("_type_name", &QPDFObjectHandle::getTypeName)
         .def(
@@ -369,6 +238,9 @@ void init_object(py::module_ &m)
         .def("__repr__", &objecthandle_repr)
         .def("__hash__",
             [](QPDFObjectHandle &self) -> py::int_ {
+                if (self.isIndirect())
+                    throw py::type_error("Can't hash indirect object");
+
                 // Objects which compare equal must have the same hash value
                 switch (self.getTypeCode()) {
                 case qpdf_object_type_e::ot_string:
@@ -804,6 +676,16 @@ void init_object(py::module_ &m)
                 }
                 return og.getInstructions();
             })
+        .def(
+            "_get_unique_resource_name",
+            [](QPDFObjectHandle &h, std::string const &prefix, int min_suffix) {
+                auto name = h.getUniqueResourceName(prefix, min_suffix);
+                return std::pair(name, min_suffix);
+            },
+            py::arg("prefix")     = "",
+            py::arg("min_suffix") = 0)
+        .def("_get_resource_names",
+            [](QPDFObjectHandle &h) { return h.getResourceNames(); })
         .def(
             "unparse",
             [](QPDFObjectHandle &h, bool resolved) -> py::bytes {
@@ -817,7 +699,10 @@ void init_object(py::module_ &m)
             [](QPDFObjectHandle &h,
                 bool dereference   = false,
                 int schema_version = 2) -> py::bytes {
-                return h.getJSON(schema_version, dereference).unparse();
+                std::string result;
+                Pl_String p("json", nullptr, result);
+                h.writeJSON(schema_version, &p, dereference);
+                return result;
             },
             py::arg("dereference")    = false,
             py::arg("schema_version") = 2); // end of QPDFObjectHandle bindings
@@ -861,7 +746,8 @@ void init_object(py::module_ &m)
         py::arg("op"));
     m.def("_Null", &QPDFObjectHandle::newNull, "Construct a PDF Null object");
 
-    py::class_<QPDFObjectHandle::ParserCallbacks, PyParserCallbacks>(m, "StreamParser")
+    py::class_<QPDFObjectHandle::ParserCallbacks, py::smart_holder, PyParserCallbacks>(
+        m, "StreamParser")
         .def(py::init<>(), "You must call ``super.__init__()`` in subclasses.")
         // LCOV_EXCL_START
         // coverage misses the virtual function call ::handleObject here.
@@ -875,10 +761,8 @@ void init_object(py::module_ &m)
 
     // Since QPDFEmbeddedFileDocumentHelper::getEmbeddedFiles returns
     // std::map<std::string, std::shared_ptr<QPDFFileSpecObjectHelper>>
-    // we must ensure that the entire QPDFObjectHelper type hierarchy is held in
-    // std::shared_ptr or repackage that interface so it does not return a
-    // std::shared_ptr<QPDFFileSpecObjectHelper>> to Python.
-    py::class_<QPDFObjectHelper, std::shared_ptr<QPDFObjectHelper>>(m, "ObjectHelper")
+    // we must use smart_holder.
+    py::class_<QPDFObjectHelper, py::smart_holder>(m, "ObjectHelper")
         .def(
             "__eq__",
             [](QPDFObjectHelper &self, QPDFObjectHelper &other) {
