@@ -6,9 +6,16 @@
 from __future__ import annotations
 
 import codecs
-from typing import Any, Container
+import sys
+from collections.abc import Container
+from typing import Any
 
-from ._core import pdf_doc_to_utf8, utf8_to_pdf_doc
+from pikepdf._core import pdf_doc_to_utf8, utf8_to_pdf_doc
+
+if sys.version_info >= (3, 12):
+    from collections.abc import Buffer
+else:
+    Buffer = Any
 
 # pylint: disable=redefined-builtin
 
@@ -16,7 +23,7 @@ from ._core import pdf_doc_to_utf8, utf8_to_pdf_doc
 # The following generates set of all Unicode code points that can be encoded in
 # pdfdoc. Since pdfdoc is 8-bit, the vast majority of code points cannot be.
 
-# Due to a bug, QPDF <= 10.5 and pikepdf < 5 had some inconsistencies around
+# Due to a bug, qpdf <= 10.5 and pikepdf < 5 had some inconsistencies around
 # PdfDocEncoding.
 PDFDOC_ENCODABLE = frozenset(
     list(range(0x00, 0x17 + 1))
@@ -104,7 +111,7 @@ def pdfdoc_encode(input: str, errors: str = 'strict') -> tuple[bytes, int]:
     raise LookupError(errors)
 
 
-def pdfdoc_decode(input: bytes, errors: str = 'strict') -> tuple[str, int]:
+def pdfdoc_decode(input: Buffer, errors: str = 'strict') -> tuple[str, int]:
     """Convert PdfDoc-encoded input into a Python str."""
     if isinstance(input, memoryview):
         input = input.tobytes()
@@ -130,7 +137,7 @@ class PdfDocCodec(codecs.Codec):
         """Implement codecs.Codec.encode for pdfdoc."""
         return pdfdoc_encode(input, errors)
 
-    def decode(self, input: bytes, errors: str = 'strict') -> tuple[str, int]:
+    def decode(self, input: Buffer, errors: str = 'strict') -> tuple[str, int]:
         """Implement codecs.Codec.decode for pdfdoc."""
         return pdfdoc_decode(input, errors)
 
